<!--

    MAX! Heizungssteuerung via Internet
    Copyright (c) 2011 eQ-3 Entwicklung GmbH. Alle Rechte vorbehalten
    @author eQ-3 Entwicklung GmbH

-->

<xsl:stylesheet version="1.0" 
xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
xmlns:integer="http://xml.apache.org/xalan/java/java.lang.Integer" 
xmlns:str="http://exslt.org/strings" 
xmlns:java_lang="http://xml.apache.org/xalan/java/java.lang" 
exclude-result-prefixes="integer java_lang">
	
	<xsl:output method="text" encoding="UTF-8" indent="yes"/>
	
	<xsl:variable name="node" select="//genericMarshallingObject/."/>
	
	<xsl:template match="//genericMarshallingObject">
		
		<xsl:message>
			<xsl:value-of select="'processing of device list started...'"/>
		</xsl:message>

		<xsl:variable name="devices">
			<xsl:for-each select="$node/b">
				<xsl:if test="
				   java_lang:Integer.parseInt(.,2) = $lengthSegmentTypeA 
				or java_lang:Integer.parseInt(.,2) = $lengthSegmentTypeB 
				or java_lang:Integer.parseInt(.,2) = $lengthSegmentTypeC">
					<xsl:message>
						<xsl:value-of select="concat('found data segment at node ', position())"/>
					</xsl:message>
					<xsl:value-of select="position()"/>
					<xsl:text>|</xsl:text>
				</xsl:if>
			</xsl:for-each>
		</xsl:variable>
		
		<xsl:message>
			<xsl:value-of select="'iterating data segments...'"/>
		</xsl:message>
		<xsl:for-each select="str:tokenize($devices, '|')">
			<xsl:variable name="pos" select="."/>
			<xsl:message>
				<xsl:value-of select="concat('processing from node position ', $pos)"/>
			</xsl:message>
			<xsl:call-template name="checkForSchemaToApply">
				<xsl:with-param name="nodePosition">
					<xsl:value-of select="$pos"/>
				</xsl:with-param>
			</xsl:call-template>
		</xsl:for-each>
		
	</xsl:template>
	
	
	<xsl:template name="checkForSchemaToApply">
		<xsl:param name="nodePosition"/>
		
		<xsl:value-of select="concat('&#xa;', 'Position: ', $nodePosition, ' - ')"/>
		
		<xsl:for-each select="$node/b">
			<xsl:if test="position() = $nodePosition">
				<xsl:variable name="statusLength">
					<xsl:value-of select="java_lang:Integer.parseInt(.,2)"/>
				</xsl:variable>
				<xsl:message>
					<xsl:value-of select="concat('reading ', $statusLength, ' more bytes')"/>
				</xsl:message>
				
				<xsl:choose>
					<xsl:when test="$statusLength = $lengthSegmentTypeA">
						<xsl:value-of select="concat('Heizungsthermostat', '&#xa;')"/>
					</xsl:when>
					<xsl:when test="$statusLength = $lengthSegmentTypeB">
						<xsl:value-of select="concat('Fensterkontakt', '&#xa;')"/>
					</xsl:when>
					<xsl:when test="$statusLength = $lengthSegmentTypeC">
						<xsl:value-of select="concat('Wandtaster', '&#xa;')"/>
					</xsl:when>
				</xsl:choose>
				
				<xsl:call-template name="applySchemaCommonData">
					<xsl:with-param name="nodePosition">
						<xsl:value-of select="$nodePosition"/>
					</xsl:with-param>
				</xsl:call-template>
				
				<xsl:choose>
					<xsl:when test="$statusLength = $lengthSegmentTypeA">
						<xsl:call-template name="applySchemaETRV">
							<xsl:with-param name="nodePosition">
								<xsl:value-of select="$nodePosition"/>
							</xsl:with-param>
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="$statusLength = $lengthSegmentTypeB">
						<xsl:call-template name="applySchemaShutterContact">
							<xsl:with-param name="nodePosition">
								<xsl:value-of select="$nodePosition"/>
							</xsl:with-param>
						</xsl:call-template>
					</xsl:when>
					<xsl:when test="$statusLength = $lengthSegmentTypeC">
						<xsl:call-template name="applySchemaPushButton">
							<xsl:with-param name="nodePosition">
								<xsl:value-of select="$nodePosition"/>
							</xsl:with-param>
						</xsl:call-template>
					</xsl:when>
				</xsl:choose>
				
				<xsl:value-of select="'________________________________________________________________________________'"/>
				
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
	
	<xsl:template name="applySchemaCommonData">
		<xsl:param name="nodePosition"/>

		<xsl:message>
			<xsl:value-of select="'applySchemaCommonData'"/>
		</xsl:message>
		
		<!-- variables -->
		<xsl:variable name="rfAddress">
			<xsl:value-of select="$node/b[$nodePosition+1]"/>
			<xsl:value-of select="$node/b[$nodePosition+2]"/>
			<xsl:value-of select="$node/b[$nodePosition+3]"/>
		</xsl:variable>
		<xsl:variable name="statusByte" select="$node/b[$nodePosition+5]"/>
		<xsl:variable name="statusValid" select="substring($statusByte, 4, 1)"/>
		<xsl:variable name="error" select="substring($statusByte, 5, 1)"/>
		<xsl:variable name="answer" select="substring($statusByte, 6, 1)"/>
		<xsl:variable name="statusInitialized" select="substring($statusByte, 7, 1)"/>

		<!-- output -->
		<xsl:value-of select="concat('Funkadresse: ', java_lang:Integer.parseInt($rfAddress,2), '&#xa;')"/>
		<xsl:value-of select="'Status: '"/>
		<xsl:choose>
			<xsl:when test="$statusValid = 0">
				<xsl:value-of select="concat('ungültig/veraltet', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('gültig', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'initialisiert: '"/>
		<xsl:choose>
			<xsl:when test="$statusInitialized = 0">
				<xsl:value-of select="concat('nein', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('ja', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'Antwort auf Funkbefehl: '"/>
		<xsl:choose>
			<xsl:when test="$answer = 0">
				<xsl:value-of select="concat('ja', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('nein', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'Fehler: '"/>
		<xsl:choose>
			<xsl:when test="$error = 0">
				<xsl:value-of select="concat('nein', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('ja, siehe Rahmentyp', '&#xa;')"/>
				<xsl:variable name="frameType">
					<xsl:value-of select="java_lang:Integer.parseInt($node/b[$nodePosition+4],2)"/>
				</xsl:variable>
				<xsl:value-of select="'Rahmentyp: '"/>
				<!-- <xsl:value-of select="concat(java_lang:Integer.toHexString($frameType), '&#xa;')"/>  -->
			</xsl:otherwise>
		</xsl:choose>
		
	</xsl:template>
	
	<xsl:template name="applySchemaETRV">
		<xsl:param name="nodePosition"/>
		
		<xsl:message>
			<xsl:value-of select="'applySchemaETRV'"/>
		</xsl:message>
		<xsl:variable name="byteOne" select="$node/b[$nodePosition+6]"/>
		<xsl:variable name="lowBattery" select="substring($byteOne, 1, 1)"/>
		<xsl:variable name="linkPartner" select="substring($byteOne, 2, 1)"/>
		<xsl:variable name="panel" select="substring($byteOne, 3, 1)"/>
		<xsl:variable name="lanGateway" select="substring($byteOne, 4, 1)"/>
		<xsl:variable name="time" select="substring($byteOne, 5, 1)"/>
		<xsl:variable name="modus" select="substring($byteOne, 7, 2)"/>
		<xsl:value-of select="'Batteriespannung: '"/>
		<xsl:choose>
			<xsl:when test="$lowBattery = 0">
				<xsl:value-of select="concat('in Ordnung', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('zu gering', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'Verknüpfungspartner: '"/>
		<xsl:choose>
			<xsl:when test="$linkPartner = 0">
				<xsl:value-of select="concat('Kommunikation in Ordnung', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('Kommunikationsstörung', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'Bedienelement: '"/>
		<xsl:choose>
			<xsl:when test="$panel = 0">
				<xsl:value-of select="concat('freigegeben', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('gesperrt', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'LAN-Gateway: '"/>
		<xsl:choose>
			<xsl:when test="$lanGateway = 0">
				<xsl:value-of select="concat('unbekannt', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('bekannt', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'Sommer-Winterzeit-Umstellung: '"/>
		<xsl:choose>
			<xsl:when test="$time = 0">
				<xsl:value-of select="concat('deaktiviert', '&#xa;')"/>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="concat('aktiv', '&#xa;')"/>
			</xsl:otherwise>
		</xsl:choose>
		<xsl:value-of select="'Temperatur-Modus: '"/>
		<xsl:choose>
			<xsl:when test="$modus = 00">
				<xsl:value-of select="concat('Auto/Wochenprogramm', '&#xa;')"/>
			</xsl:when>
			<xsl:when test="$modus = 01">
				<xsl:value-of select="concat('Manuell', '&#xa;')"/>
			</xsl:when>
			<xsl:when test="$modus = 10">
				<xsl:value-of select="concat('Party/Urlaub', '&#xa;')"/>
			</xsl:when>
			<xsl:when test="$modus = 11">
				<xsl:value-of select="concat('Boost', '&#xa;')"/>
			</xsl:when>
		</xsl:choose>
		</xsl:template>
	
	<xsl:template name="applySchemaShutterContact">
		<xsl:param name="nodePosition"/>

		<xsl:message>
			<xsl:value-of select="'applySchemaShutterContact'"/>
		</xsl:message>

	</xsl:template>
	
	<xsl:template name="applySchemaPushButton">
		<xsl:param name="nodePosition"/>

		<xsl:message>
			<xsl:value-of select="'applySchemaPushButton'"/>
		</xsl:message>

	</xsl:template>
	
	<xsl:variable name="lengthSegmentTypeA" select="11"/>
	<xsl:variable name="lengthSegmentTypeB" select="6"/>
	<xsl:variable name="lengthSegmentTypeC" select="7"/>
	
</xsl:stylesheet>